@@ -19,6 +19,7 @@
 //! [RFC 1990 stabilizes](https://github.com/rust-lang/rust/issues/44732).
 
 #[cfg(feature = "std")]
+#[macro_use]
 extern crate std;
 
 use core::ops::{BitAnd, BitAndAssign, BitOr, BitOrAssign, BitXor, BitXorAssign, Neg, Not};
@@ -76,12 +77,7 @@ impl From<Choice> for bool {
     #[inline]
     fn from(source: Choice) -> bool {
         debug_assert!(source.0 == 0u8 || source.0 == 1u8);
-
-        match source.0 {
-            0 => return false,
-            1 => return true,
-            _ => unsafe { ::core::hint::unreachable_unchecked() },
-        }
+        source.0 != 0
     }
 }
 
@@ -292,23 +288,15 @@ generate_integer_equal!(u8, i8, 8);
 generate_integer_equal!(u16, i16, 16);
 generate_integer_equal!(u32, i32, 32);
 generate_integer_equal!(u64, i64, 64);
+#[cfg(feature = "i128")]
 generate_integer_equal!(u128, i128, 128);
+generate_integer_equal!(usize, isize, ::core::mem::size_of::<usize>() * 8);
 
-/// Select one of two inputs according to a `Choice` in constant time.
+/// A type which can be conditionally selected in constant time.
 ///
-/// # Examples
-///
-/// ```
-/// # use subtle;
-/// use subtle::ConditionallySelectable;
-/// use subtle::Choice;
-/// let a: i32 = 5;
-/// let b: i32 = 13;
-///
-/// assert_eq!(i32::conditional_select(&a, &b, Choice::from(0)), a);
-/// assert_eq!(i32::conditional_select(&a, &b, Choice::from(1)), b);
-/// ```
-pub trait ConditionallySelectable {
+/// This trait also provides generic implementations of conditional
+/// assignment and conditional swaps.
+pub trait ConditionallySelectable: Copy {
     /// Select `a` or `b` according to `choice`.
     ///
     /// # Returns
@@ -317,8 +305,80 @@ pub trait ConditionallySelectable {
     /// * `b` if `choice == Choice(1)`.
     ///
     /// This function should execute in constant time.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # extern crate subtle;
+    /// use subtle::ConditionallySelectable;
+    /// #
+    /// # fn main() {
+    /// let x: u8 = 13;
+    /// let y: u8 = 42;
+    ///
+    /// let z = u8::conditional_select(&x, &y, 0.into());
+    /// assert_eq!(z, x);
+    /// let z = u8::conditional_select(&x, &y, 1.into());
+    /// assert_eq!(z, y);
+    /// # }
+    /// ```
     #[inline]
     fn conditional_select(a: &Self, b: &Self, choice: Choice) -> Self;
+
+    /// Conditionally assign `other` to `self`, according to `choice`.
+    ///
+    /// This function should execute in constant time.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # extern crate subtle;
+    /// use subtle::ConditionallySelectable;
+    /// #
+    /// # fn main() {
+    /// let mut x: u8 = 13;
+    /// let mut y: u8 = 42;
+    ///
+    /// x.conditional_assign(&y, 0.into());
+    /// assert_eq!(x, 13);
+    /// x.conditional_assign(&y, 1.into());
+    /// assert_eq!(x, 42);
+    /// # }
+    /// ```
+    #[inline]
+    fn conditional_assign(&mut self, other: &Self, choice: Choice) {
+        *self = Self::conditional_select(self, other, choice);
+    }
+
+    /// Conditionally swap `self` and `other` if `choice == 1`; otherwise,
+    /// reassign both unto themselves.
+    ///
+    /// This function should execute in constant time.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # extern crate subtle;
+    /// use subtle::ConditionallySelectable;
+    /// #
+    /// # fn main() {
+    /// let mut x: u8 = 13;
+    /// let mut y: u8 = 42;
+    ///
+    /// u8::conditional_swap(&mut x, &mut y, 0.into());
+    /// assert_eq!(x, 13);
+    /// assert_eq!(y, 42);
+    /// u8::conditional_swap(&mut x, &mut y, 1.into());
+    /// assert_eq!(x, 42);
+    /// assert_eq!(y, 13);
+    /// # }
+    /// ```
+    #[inline]
+    fn conditional_swap(a: &mut Self, b: &mut Self, choice: Choice) {
+        let t: Self = *a;
+        a.conditional_assign(&b, choice);
+        b.conditional_assign(&t, choice);
+    }
 }
 
 macro_rules! to_signed_int {
@@ -362,7 +422,25 @@ macro_rules! generate_integer_conditional_select {
                 // if choice = 0, mask = (-0) = 0000...0000
                 // if choice = 1, mask = (-1) = 1111...1111
                 let mask = -(choice.unwrap_u8() as to_signed_int!($t)) as $t;
-                a ^ ((mask) & (a ^ b))
+                a ^ (mask & (a ^ b))
+            }
+
+            #[inline]
+            fn conditional_assign(&mut self, other: &Self, choice: Choice) {
+                // if choice = 0, mask = (-0) = 0000...0000
+                // if choice = 1, mask = (-1) = 1111...1111
+                let mask = -(choice.unwrap_u8() as to_signed_int!($t)) as $t;
+                *self ^= mask & (*self ^ *other);
+            }
+
+            #[inline]
+            fn conditional_swap(a: &mut Self, b: &mut Self, choice: Choice) {
+                // if choice = 0, mask = (-0) = 0000...0000
+                // if choice = 1, mask = (-1) = 1111...1111
+                let mask = -(choice.unwrap_u8() as to_signed_int!($t)) as $t;
+                let t = mask & (*a ^ *b);
+                *a ^= t;
+                *b ^= t;
             }
          }
     )*)
@@ -372,14 +450,16 @@ generate_integer_conditional_select!(  u8   i8);
 generate_integer_conditional_select!( u16  i16);
 generate_integer_conditional_select!( u32  i32);
 generate_integer_conditional_select!( u64  i64);
+#[cfg(feature = "i128")]
 generate_integer_conditional_select!(u128 i128);
 
 /// A type which can be conditionally negated in constant time.
 ///
 /// # Note
 ///
-/// A generic implementation of `ConditionallyNegatable` is provided for types
-/// which are `ConditionallyNegatable + Neg`.
+/// A generic implementation of `ConditionallyNegatable` is provided
+/// for types `T` which are `ConditionallySelectable` and have `Neg`
+/// implemented on `&T`.
 pub trait ConditionallyNegatable {
     /// Negate `self` if `choice == Choice(1)`; otherwise, leave it
     /// unchanged.
@@ -391,7 +471,7 @@ pub trait ConditionallyNegatable {
 
 impl<T> ConditionallyNegatable for T
 where
-    T: ConditionallyAssignable,
+    T: ConditionallySelectable,
     for<'a> &'a T: Neg<Output = T>,
 {
     #[inline]
@@ -401,172 +481,3 @@ where
         self.conditional_assign(&self_neg, choice);
     }
 }
-
-/// A type which can be conditionally assigned in constant time.
-pub trait ConditionallyAssignable {
-    /// Conditionally assign `other` to `self`, according to `choice`.
-    ///
-    /// This function should execute in constant time.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// # extern crate subtle;
-    /// use subtle::ConditionallyAssignable;
-    /// #
-    /// # fn main() {
-    /// let mut x: u8 = 13;
-    /// let y:     u8 = 42;
-    ///
-    /// x.conditional_assign(&y, 0.into());
-    /// assert_eq!(x, 13);
-    /// x.conditional_assign(&y, 1.into());
-    /// assert_eq!(x, 42);
-    /// # }
-    /// ```
-    ///
-    #[inline]
-    fn conditional_assign(&mut self, other: &Self, choice: Choice);
-}
-
-impl<T> ConditionallyAssignable for T
-where
-    T: ConditionallySelectable,
-{
-    #[inline]
-    fn conditional_assign(&mut self, other: &Self, choice: Choice) {
-        *self = T::conditional_select(self, other, choice);
-    }
-}
-
-/// A type which is conditionally swappable in constant time.
-pub trait ConditionallySwappable {
-    /// Conditionally swap `self` and `other` if `choice == 1`; otherwise,
-    /// reassign both unto themselves.
-    ///
-    /// # Note
-    ///
-    /// This trait is generically implemented for any type which implements
-    /// `ConditionallyAssignable + Copy`.
-    #[inline]
-    fn conditional_swap(&mut self, other: &mut Self, choice: Choice);
-}
-
-impl<T> ConditionallySwappable for T
-where
-    T: ConditionallyAssignable + Copy,
-{
-    #[inline]
-    fn conditional_swap(&mut self, other: &mut T, choice: Choice) {
-        let temp: T = *self;
-        self.conditional_assign(&other, choice);
-        other.conditional_assign(&temp, choice);
-    }
-}
-
-#[cfg(test)]
-mod test {
-    use super::*;
-
-    #[test]
-    #[should_panic]
-    fn slices_equal_different_lengths() {
-        let a: [u8; 3] = [0, 0, 0];
-        let b: [u8; 4] = [0, 0, 0, 0];
-
-        assert_eq!((&a).ct_eq(&b).unwrap_u8(), 1);
-    }
-
-    #[test]
-    fn slices_equal() {
-        let a: [u8; 8] = [1, 2, 3, 4, 5, 6, 7, 8];
-        let b: [u8; 8] = [1, 2, 3, 4, 4, 3, 2, 1];
-
-        let a_eq_a = (&a).ct_eq(&a);
-        let a_eq_b = (&a).ct_eq(&b);
-
-        assert_eq!(a_eq_a.unwrap_u8(), 1);
-        assert_eq!(a_eq_b.unwrap_u8(), 0);
-
-        let c: [u8; 16] = [0u8; 16];
-
-        let a_eq_c = (&a).ct_eq(&c);
-        assert_eq!(a_eq_c.unwrap_u8(), 0);
-    }
-
-    #[test]
-    fn conditional_assign_i32() {
-        let mut a: i32 = 5;
-        let b: i32 = 13;
-
-        a.conditional_assign(&b, 0.into());
-        assert_eq!(a, 5);
-        a.conditional_assign(&b, 1.into());
-        assert_eq!(a, 13);
-    }
-
-    #[test]
-    fn conditional_assign_i64() {
-        let mut c: i64 = 2343249123;
-        let d: i64 = 8723884895;
-
-        c.conditional_assign(&d, 0.into());
-        assert_eq!(c, 2343249123);
-        c.conditional_assign(&d, 1.into());
-        assert_eq!(c, 8723884895);
-    }
-
-    macro_rules! generate_integer_conditional_select_tests {
-        ($($t:ty)*) => ($(
-            let x: $t = 0;  // all 0 bits
-            let y: $t = !0; // all 1 bits
-
-            assert_eq!(<$t>::conditional_select(&x, &y, 0.into()), 0);
-            assert_eq!(<$t>::conditional_select(&x, &y, 1.into()), y);
-        )*)
-    }
-
-    #[test]
-    fn integer_conditional_select() {
-        generate_integer_conditional_select_tests!(u8 u16 u32 u64 u128);
-        generate_integer_conditional_select_tests!(i8 i16 i32 i64 i128);
-    }
-
-    #[test]
-    fn custom_conditional_select_i16() {
-        let x: i16 = 257;
-        let y: i16 = 514;
-
-        assert_eq!(i16::conditional_select(&x, &y, 0.into()), 257);
-        assert_eq!(i16::conditional_select(&x, &y, 1.into()), 514);
-    }
-
-    macro_rules! generate_integer_equal_tests {
-        ($($t:ty),*) => ($(
-            let y: $t = 0;  // all 0 bits
-            let z: $t = !0; // all 1 bits
-
-            let x = z;
-
-            assert_eq!(x.ct_eq(&y).unwrap_u8(), 0);
-            assert_eq!(x.ct_eq(&z).unwrap_u8(), 1);
-        )*)
-    }
-
-    #[test]
-    fn integer_equal() {
-        generate_integer_equal_tests!(u8, u16, u32, u64, u128);
-        generate_integer_equal_tests!(i8, i16, i32, i64, i128);
-    }
-
-    #[test]
-    fn choice_into_bool() {
-        let choice_true: bool = Choice::from(1).into();
-
-        assert!(choice_true);
-
-        let choice_false: bool = Choice::from(0).into();
-
-        assert!(!choice_false);
-    }
-}
